<?php

class User{

	public $user_params = [
		'internal_id' => null,
		'user_id' => 0,
		'user_name' => '',
		'user_slug' => '',
		'level' => null,
		'rank' => null,
		'battle_id' => null,
		'realm' => ''
	];

	private $user_table = '#__wow_guild_user_user';

	private $db;

	public function __construct(){
		$this->db = JFactory::getDbo();
	}

	public function isUserExistsByNameAndRealm($name, $realm){
		$query = $this->db->getQuery(true);

		$query->select('count(*) as count');
		$query->from($this->db->quoteName($this->user_table));
		$query->where(
			[
				$this->db->quoteName('user_name') . ' = ' . $this->db->quote($name),
				$this->db->quoteName('realm') . ' = ' . $this->db->quote($realm),
			]
		);

		$this->db->setQuery($query);
		$results = $this->db->loadObject();

		return ($results->count > 0);
	}

	public function isUserExistsByUserId($id){
		$query = $this->db->getQuery(true);

		$query->select('count(*) as count');
		$query->from($this->db->quoteName($this->user_table));
		$query->where($this->db->quoteName('user_id') . ' = ' . $this->db->quote($id));

		$this->db->setQuery($query);
		$results = $this->db->loadObject();

		return ($results->count > 0) ? false : true;
	}

	private function getJoomlaUserIdByName($name){
		$query = $this->db->getQuery(true);

		$query->select('id');
		$query->from($this->db->quoteName('#__users'));
		$query->where('LOWER('.$this->db->quoteName('username').')' . ' = ' . $this->db->quote(strtolower($name)));

		$this->db->setQuery($query);
		$results = $this->db->loadObject();

		return ($results == '') ? 0 : $results->id;
	}

	private function update(){
		$query = $this->db->getQuery(true);

		$fields = [];
		foreach($this->user_params as $key => $params){
			if($key !== 'internal_id'){
				$fields[] = $this->db->quoteName($key) . ' = ' . (is_int($params) ? $params : $this->db->quote($params));
			}
		}

		$conditions = $this->db->quoteName('battle_id') . ' = ' . $this->user_params['battle_id'];
		$query->update($this->db->quoteName($this->user_table))->set($fields)->where($conditions);
		$this->db->setQuery($query);
		$this->db->execute();

		/**
		 * get Internal ID
		 */
		$query = $this->db->getQuery(true);
		$query->select('id');
		$query->from($this->db->quoteName($this->user_table));
		$query->where($conditions);
		$this->db->setQuery($query);
		$results = $this->db->loadObject();

		$this->user_params['internal_id'] = $results->id;
	}

	private function insert(){
		$query = $this->db->getQuery(true);

		$values = [];
		foreach($this->user_params as $key => $params){
			if($key !== 'internal_id') {
				$values[$key] = is_int($params) ? $params : $this->db->quote($params);
			}
		}

		$values['created'] = $this->db->quote(date('Y-m-d H:i:s'));

		$query
			->insert($this->db->quoteName($this->user_table))
			->columns($this->db->quoteName(array_keys($values)))
			->values(implode(',', $values));

		$this->db->setQuery($query);
		$this->db->execute();

		$this->user_params['internal_id'] = $this->db->insertid();
	}

	public function addJoomlaUserId(){
		if($id = $this->getJoomlaUserIdByName($this->get('user_name'))){
			$this->set('user_id', $id);
		}
	}

	public function save(){
		if($this->isUserExistsByNameAndRealm($this->get('user_name'), $this->get('realm'))){
			$this->update();
		}else{
			$this->insert();
		}
		return $this;
	}

	function getNameSlug($name){
		return urlencode(strtolower($name));
	}

	public function get($name){
		return $this->user_params[$name];
	}

	public function set($name, $value){
		if($name === 'user_name'){
			$this->user_params['user_slug'] = $this->getNameSlug($value);
		}

		$this->user_params[$name] = $value;
		return $this;
	}

}
