<?php
/**
 * @package     Joomla.Plugin
 * @subpackage  Captcha
 *
 * @copyright   (C) 2011 Open Source Matters, Inc. <https://www.joomla.org>
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

require_once 'User.php';
require_once 'Achievements.php';

class PlgContentUsersync extends JPlugin
{

	protected $autoloadLanguage = true;

	private $realm_slug;
	private $guild_slug;
	private $region;
	private $locale;
	private $token;
	private $link_wow_user;
	private $sync_achievements;
	private $achievements_list;

	public function __construct(&$subject, $config = array()){

		parent::__construct($subject, $config);

		$this->realm_slug = $this->params->get('realm_slug');
		$this->guild_slug = $this->params->get('guild_slug');
		$this->region = $this->params->get('region');
		$this->locale = $this->params->get('locale');
		$this->link_wow_user = $this->params->get('link_wow_user');
		$this->sync_achievements = $this->params->get('sync_achievements');
		$this->achievements_list = $this->params->get('achievements_list');
	}

	function isPluginBattlenetActive(){
		return !(empty(JPluginHelper::getPlugin('system', 'battlenet')));
	}

	function getToken(){
		$battlenet_config = JPluginHelper::getPlugin('system', 'battlenet');
		$token = json_decode($battlenet_config->params)->client_token;
		return $token;
	}

	function requestGetGuildRoster(){

		$curl = curl_init();
		$response = new stdClass();

		curl_setopt_array($curl, array(
			CURLOPT_URL => 'https://'.$this->region.'.api.blizzard.com/data/wow/guild/'.$this->realm_slug.'/'.$this->guild_slug.'/roster?namespace=profile-'.$this->region.'&access_token='.$this->token.'&locale='.$this->locale,
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_ENCODING => '',
			CURLOPT_MAXREDIRS => 10,
			CURLOPT_TIMEOUT => 0,
			CURLOPT_FOLLOWLOCATION => true,
			CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
			CURLOPT_CUSTOMREQUEST => 'GET',
		));

		if(curl_exec($curl) === false){
			$response->error = curl_error($curl);
		}else{
			$response = json_decode(curl_exec($curl));
		}

		curl_close($curl);

		/**
		 * stdClass Object
		(
		[access_token] => USxBBl7P6336Kk1cwVhlI9B8d3pc9RR4vT
		[token_type] => bearer
		[expires_in] => 86399
		[sub] => aa0c7df71dfa4e9194998c6797fa61f4
		)
		 */
		return $response;
	}

	public function requestGetCharacterAchievements($user){
		$curl = curl_init();

		$response = new stdClass();

		curl_setopt_array($curl, array(
			CURLOPT_URL => 'https://'.$this->region.'.api.blizzard.com/profile/wow/character/'.$user->get('realm').'/'.$user->get('user_slug').'/achievements?namespace=profile-'.$this->region.'&access_token='.$this->token.'&locale='.$this->locale,
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_ENCODING => '',
			CURLOPT_MAXREDIRS => 10,
			CURLOPT_TIMEOUT => 0,
			CURLOPT_FOLLOWLOCATION => true,
			CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
			CURLOPT_CUSTOMREQUEST => 'GET',
		));

		if(curl_exec($curl) === false){
			$response->error = curl_error($curl);
		}else{
			$response = json_decode(curl_exec($curl));
		}

		curl_close($curl);
		return $response;
	}

	public function cronUpdate(){

		if($this->isPluginBattlenetActive()){

			$this->token = $this->getToken();
			$guild_roster = $this->requestGetGuildRoster();

			if(!isset($guild_roster->error)){

				$members = $guild_roster->members;

				foreach($members as $member){

					$character = $member->character;

					$user = new User();
					$user->set('user_name', $character->name);
					$user->set('level', $character->level);
					$user->set('rank', $member->rank);
					$user->set('battle_id', $character->id);
					$user->set('realm', $character->realm->slug);

					if($this->link_wow_user == 1){
						$user->addJoomlaUserId();
					}

					$user->save();

					if(
						$this->sync_achievements != 0 &&
						$user->get('user_id') != 0
					){
						$achievement = new Achievements($user);
						$achievements = $this->requestGetCharacterAchievements($user);
						$achievement_list = explode('|', $this->achievements_list);

						foreach($achievements->achievements as $key => $value){
							if(
								($this->sync_achievements == 1) || // All
								($this->sync_achievements == 2 && in_array($value->id, $achievement_list)) // Selected
							){

								if(!isset($value->criteria->is_completed)){
									$completed = 0;
								}else{
									$completed = ($value->criteria->is_completed) ? 1 : 0;
								}

								$achievement->addAchievement(
									$value->id,
									$value->achievement->name,
									$completed
								);
							}
						}

						$achievement->save();
					}

				}
			}else{
				echo $guild_roster->error;
			}
		}
	}

}
